@@ -1,10 +1,11 @@
+use std::thread::JoinHandle;
+use std::time::Instant;
 use std::{
     io::{stdout, Write},
     sync::mpsc::{channel, Sender, TryRecvError},
     thread,
     time::Duration,
 };
-use std::time::Instant;
 
 pub use crate::utils::spinner_names::SpinnerNames as Spinners;
 use crate::utils::spinners_data::SPINNERS as SpinnersMap;
@@ -12,12 +13,16 @@ use crate::utils::spinners_data::SPINNERS as SpinnersMap;
 mod utils;
 
 pub struct Spinner {
-    sender: Sender<(Instant, Option<String>)>
+    sender: Sender<(Instant, Option<String>)>,
+    join: Option<JoinHandle<()>>,
 }
 
 impl Drop for Spinner {
     fn drop(&mut self) {
-        self.sender.send((Instant::now(),  None)).unwrap();
+        if self.join.is_some() {
+            self.sender.send((Instant::now(), None)).unwrap();
+            self.join.take().unwrap().join().unwrap();
+        }
     }
 }
 
@@ -58,7 +63,7 @@ impl Spinner {
 
         let (sender, recv) = channel::<(Instant, Option<String>)>();
 
-        thread::spawn(move || 'outer: loop {
+        let join = thread::spawn(move || 'outer: loop {
             let mut stdout = stdout();
             for frame in spinner_data.frames.iter() {
                 let (do_stop, stop_time, stop_symbol) = match recv.try_recv() {
@@ -67,19 +72,18 @@ impl Spinner {
                     Err(TryRecvError::Empty) => (false, None, None),
                 };
 
-                let frame = stop_symbol.unwrap_or(frame.to_string());
+                let frame = stop_symbol.unwrap_or_else(|| frame.to_string());
                 match start_time {
                     None => {
                         print!("\r{} {}", frame, message);
                     }
                     Some(start_time) => {
-                        let now = stop_time.unwrap_or(Instant::now());
+                        let now = stop_time.unwrap_or_else(Instant::now);
                         let duration = now.duration_since(start_time).as_secs_f64();
                         print!("\r{}{:>10.3} s\t{}", frame, duration, message);
                     }
                 }
 
-
                 stdout.flush().unwrap();
 
                 if do_stop {
@@ -90,11 +94,12 @@ impl Spinner {
             }
         });
 
-        Self {sender}
+        Self {
+            sender,
+            join: Some(join),
+        }
     }
 
-    // TODO: Add update message function
-
     /// Stops the spinner
     ///
     /// Stops the spinner that was created with the [`Spinner::new`] function.
@@ -113,12 +118,12 @@ impl Spinner {
     /// ```
     /// use spinners::{Spinner, Spinners};
     ///
-    /// let sp = Spinner::new(Spinners::Dots, "Loading things into memory...".into());
+    /// let mut sp = Spinner::new(Spinners::Dots, "Loading things into memory...".into());
     ///
     /// sp.stop();
     /// ```
-    pub fn stop(self) {
-        self.stop_inner(Instant::now(),  None);
+    pub fn stop(&mut self) {
+        self.stop_inner(Instant::now(), None);
     }
 
     /// Stop with a symbol that replaces the spinner
@@ -132,7 +137,7 @@ impl Spinner {
     /// ```
     /// use spinners::{Spinner, Spinners};
     ///
-    /// let sp = Spinner::new(Spinners::Dots, "Loading things into memory...".into());
+    /// let mut sp = Spinner::new(Spinners::Dots, "Loading things into memory...".into());
     ///
     /// sp.stop_with_symbol("🗸");
     /// ```
@@ -142,11 +147,11 @@ impl Spinner {
     /// ```
     /// use spinners::{Spinner, Spinners};
     ///
-    /// let sp = Spinner::new(Spinners::Dots, "Loading things into memory...".into());
+    /// let mut sp = Spinner::new(Spinners::Dots, "Loading things into memory...".into());
     ///
     /// sp.stop_with_symbol("\x1b[32m🗸\x1b[0m");
     /// ```
-    pub fn stop_with_symbol(self, symbol: &str) {
+    pub fn stop_with_symbol(&mut self, symbol: &str) {
         self.stop_inner(Instant::now(), Some(symbol.to_owned()));
         println!();
     }
@@ -160,11 +165,11 @@ impl Spinner {
     /// ```
     /// use spinners::{Spinner, Spinners};
     ///
-    /// let sp = Spinner::new(Spinners::Dots, "Loading things into memory...".into());
+    /// let mut sp = Spinner::new(Spinners::Dots, "Loading things into memory...".into());
     ///
     /// sp.stop_with_newline();
     /// ```
-    pub fn stop_with_newline(self) {
+    pub fn stop_with_newline(&mut self) {
         self.stop();
         println!();
     }
@@ -178,19 +183,19 @@ impl Spinner {
     /// ```
     /// use spinners::{Spinner, Spinners};
     ///
-    /// let sp = Spinner::new(Spinners::Dots, "Loading things into memory...".into());
+    /// let mut sp = Spinner::new(Spinners::Dots, "Loading things into memory...".into());
     ///
     /// sp.stop_with_message("Finished loading things into memory!".into());
     /// ```
-    pub fn stop_with_message(self, msg: String) {
+    pub fn stop_with_message(&mut self, msg: String) {
         self.stop();
         print!("\r{}", msg);
     }
 
-    fn stop_inner(self, stop_time: Instant, stop_symbol: Option<String>) {
+    fn stop_inner(&mut self, stop_time: Instant, stop_symbol: Option<String>) {
         self.sender
             .send((stop_time, stop_symbol))
             .expect("Could not stop spinner thread.");
+        self.join.take().unwrap().join().unwrap();
     }
 }
-
